@@ -1,13 +1,15 @@
 import argparse
+import ast
 import json
 import logging
+from enum import Enum
 
 from lxml import etree as ET
 from lxml import etree
+from lxml.etree import Element, _Element
 import lxml
 import os
 import re
-import sys
 import traceback
 import urllib.request
 
@@ -29,6 +31,7 @@ from py4ami.wikimedia import WikidataSparql, WikidataLookup, WikidataPage
 DICTIONARY = "dictionary"
 ENTRY = "entry"
 IMAGE = "image"
+RAW = "raw"
 TITLE = "title"
 WIKIPEDIA = "wikipedia"
 WIKIDATA = "wikidata"
@@ -69,6 +72,11 @@ LANG_UR = "ur"
 
 logger = logging.getLogger("ami_dict")
 
+"""
+work with lxml Elements as far as possible, and only wrap as Ami* objects when thos functions
+are needed. Gradually increasing specific names (e.g. _lxml_entry). I know it's not pythonic
+but it stops me making errors
+"""
 
 class AmiEntry:
     TAG = "entry"
@@ -97,8 +105,14 @@ class AmiEntry:
         ami_entry.element = entry_element
         return ami_entry
 
+# AmiEntry
+
     @classmethod
-    def create_from_term(cls, term):
+    def create_lxml_entry_from_term(cls, term):
+        """create lxml entry element
+        :param term: creates @term attribute
+        :return: lxml entry element with @term
+        """
         assert term is not None
         entry_element = lxml.etree.Element(ENTRY)
         entry_element.attrib[TERM] = term
@@ -113,13 +127,15 @@ class AmiEntry:
         return self.element.get(TERM)
 
     @property
-    def get_wikidata_id(self):
-        return self.element.get(WIKIDATA)
+    def wikidata_id(self):
+        return self.element.get(WIKIDATA_ID)
 
     @property
     def get_wikipedia_page(self):
         return self.element.get(WIKIPEDIA)
 
+    # AmiEntry
+
     def get_synonyms(self):
         """list of child synonym objects"""
         synonyms = [] if self.element is None else self.element.xpath("./" + AmiSynonym.TAG)
@@ -139,15 +155,24 @@ class AmiEntry:
         term = self.element.attrib[TERM]
         return term
 
-    @classmethod
-    def add_name(cls, entry_element, name):
-        assert entry_element is not None
-        assert entry_element.tag == ENTRY
-        entry_element.attrib[NAME] = name
+    # AmiEntry
 
     @classmethod
-    def get_name(cls, entry):
-        return AmiEntry.get_attribute_value(entry, NAME)
+    def add_name(cls, lxml_entry_element, name):
+        assert type(lxml_entry_element) is _Element, f"found {type(lxml_entry_element)}"
+        assert lxml_entry_element is not None
+        assert lxml_entry_element.tag == ENTRY
+        lxml_entry_element.attrib[NAME] = name
+
+    def set_name(self, name):
+        AmiEntry.add_name(self.element, name)
+
+    # @classmethod
+    # def get_name(cls, entry):
+    #     return AmiEntry.get_attribute_value(entry, NAME)
+
+    def get_name(self):
+        return AmiEntry.get_attribute_value(self.element, NAME)
 
     @classmethod
     def get_attribute_value(cls, element, attname):
@@ -162,8 +187,7 @@ class AmiEntry:
             raise AMIDictError(f"wikidata id {idx} must be Qddd... or Pddd...")
         self.set_attribute(WIKIDATA_ID, idx)
 
-    def wikidata_id(self):
-        return self.get_attribute_value(WIKIDATA_ID)
+    # AmiEntry
 
     def set_description(self, desc):
         """set description attribute, can be anything"""
@@ -177,6 +201,8 @@ class AmiEntry:
         self.check_valid_attributes()
         self.check_valid_children()
 
+    # AmiEntry
+
     def check_valid_attributes(self):
         attribute_names = list(self.element.attrib)
         assert attribute_names is not None
@@ -193,6 +219,8 @@ class AmiEntry:
             for child in self.element:
                 assert child.tag in self.ELEMENT_CHILD_TAGS
 
+    # AmiEntry
+
     def get_synonyms(self):
         """create AmiSynonyms from <synonym> children
         :return: list of AmiSynonyms created from children
@@ -203,6 +231,36 @@ class AmiEntry:
             ami_synonyms.append(AmiSynonym.create_from_element(synonym_element))
         return ami_synonyms
 
+    def get_raw_child_wikidata_ids(self):
+        """
+        for single child of form <raw wikidataID = [Q1, Q2...]
+        :return: list of ids, else []
+        """
+        raw_child = self.get_single_raw_child_element()
+        if raw_child is None:
+            return []
+        wikidata_id_att = raw_child.get(WIKIDATA_ID)
+        wikidata_ids = ast.literal_eval(wikidata_id_att) if wikidata_id_att else []
+        return wikidata_ids
+
+    def get_single_raw_child_element(self):
+        """
+        :return: <raw> child of element if exactly 1, else None
+        """
+        raw_children = self.get_raw_child_elements()
+        raw_child = raw_children[0] if len(raw_children) == 1 else None
+        return raw_child
+
+    # AmiEntry
+
+    def get_raw_child_elements(self):
+        """
+        get all <raw> child elements of AmiEntry.element
+        :return: list of elements (may be zero-length never None)
+        """
+        raw_children = self.element.findall(RAW)
+        return raw_children
+
     def validate_attribute(self, att_name):
         if not att_name:
             logger.warning(f"attribute name is None")
@@ -221,6 +279,107 @@ class AmiEntry:
             logging.warning(f"{WIKIDATA_ID} bad value: {value}")
         return mm
 
+    # AmiEntry
+
+    @classmethod
+    def find_name_to_wikidata_match(cls, wikidata_pages, wikidata_title=None):
+        """
+        matches wikidata pages against wikidata_title
+        (Later may have fuzzy matching)
+        :param wikidata_pages: list of pages
+        :param wikidata_title: title to match
+        :return: list of pages that match
+        """
+        matched_pages = []
+        for wikidata_page in wikidata_pages:
+            title = wikidata_page.get_title()
+            if wikidata_page.title_matches(wikidata_title):
+                # print(f"exact match for {wikidata_title} is {wikidata_page.get_id()}")
+                matched_pages.append(wikidata_page)
+        return matched_pages
+
+    @classmethod
+    def get_wikidata_pages_for_ids(cls, wikidata_ids):
+        """
+        get WikidataPages for list of PQids
+        :param wikidata_ids: list of PQitems
+        :return: list of corresponding WikidataPages
+        """
+        wikidata_pages = []
+        for id in wikidata_ids:
+            wikidata_page = WikidataPage(pqitem=id)
+            wikidata_pages.append(wikidata_page)
+        return wikidata_pages
+
+    # AmiEntry
+
+    def get_wikidata_id(self, empty_to_none=True):
+        """gets value of wikidataID attribute
+        :param empty_to_none: if True treats "" as None
+        :return: raw value or None
+        """
+        return self.element.get(WIKIDATA_ID)
+
+    @classmethod
+    def get_wikidata_ids_for_entries(cls, entries, empty_to_none=True):
+        """
+        list of wikidata_ids for list of entries
+        may contain None values if no wikidata_id
+        :param entries: list of entries to process
+        :param empty_to_none: if true treat "" as None
+        :return: list of wikidata IDs (may be [])
+        """
+
+        return [AmiEntry.create_from_element(entry).get_wikidata_id(empty_to_none=empty_to_none) for entry in entries] if entries else []
+
+    # AmiEntry
+
+    @classmethod
+    def get_terms_for_lxml_entries(cls, lxml_entries):
+        """
+        Convenience method
+        get list of @term's for list of AmiEntries
+        USABLE
+        :param entries: list of entries to process
+        :return: list of terms (may be [])
+        """
+        if len(lxml_entries) == 0:
+            return []
+        type1 = type(lxml_entries[0])
+        assert type1 is _Element, f"found {type1}"
+
+        return [lxml_entry.get(TERM) for lxml_entry in lxml_entries]
+
+    def get_wikidata_pages_from_raw_wikidata_ids_matching_wikidata_page_title(self):
+        """
+        use entry name to search for wikidata pages (from raw@wikidataID child ids) whose title/label == name
+        <entry term="GHG" name="Greenhouse Gas" >
+            <raw wikidataID="['Q167336', 'Q3588927', 'Q925312', 'Q57584895', 'Q110612403', 'Q112192791', 'Q140182']"/>
+        </entry>
+        Object: find all wikidata pages whose titles match the `name`s in the raw wikidata pages
+        1) retrieve the wikidata pages with the given wikidataIDs
+        2) find each of their titles/labels
+        3) check the entry name ("greenhouse gas") against the titles (case-insensitive, exact match)
+        4) return the pages which match
+        USABLE
+
+        :return: list of pages with title (may be empty)
+        """
+        raw_wikidata_ids = self.get_raw_child_wikidata_ids()
+        if not raw_wikidata_ids:
+            return []
+        wikidata_pages = AmiEntry.get_wikidata_pages_for_ids(raw_wikidata_ids)
+        name = self.get_name().lower()
+        matched_pages = AmiEntry.find_name_to_wikidata_match(wikidata_pages, wikidata_title=name)
+        return matched_pages
+
+    @classmethod
+    def create_from_elements(cls, _entries):
+        """create a list of AmiEntry's wrapping a list of lxml elements"""
+        ami_entries = [AmiEntry.create_from_element(_element) for _element in _entries]
+        return ami_entries
+
+
 
 class AmiDictionary:
     """wrapper for an ami dictionary including search flags
@@ -273,21 +432,33 @@ class AmiDictionary:
     #    class AmiDictionary:
 
     @classmethod
-    def create_from_xml_file(cls, xml_file, title=None, ignorecase=False):
+    def create_from_xml_file(cls, xml_file,
+                             title=None,
+                             ignorecase=False):
+        """
+        reads a dictionary XML file, creates and returns the AMIDictionary from it
+        uses AmiDictionary.read_dictionary_from_xml_file()
+        :param xml_file: file containing an AMI Dictionary as XML
+        :param title: Obsolete
+        :param ignorecase: see AmiDictionary.create_from_xml_object
+        :returns: AmiDictionary object or null
+        """
         if not os.path.exists(xml_file):
-            raise IOError("cannot find path " + str(xml_file))
-        dictionary = cls.read_dictionary_from_xml_file(xml_file, ignorecase=ignorecase)
-        file_title = Path(xml_file).stem
-        if not title:
-            title = file_title
-        print(f"title {file_title}")
+            logging.warning("cannot find dictionary path " + str(xml_file))
+            return None
+        xml_tree = ET.parse(str(xml_file), parser=ET.XMLParser(encoding="utf-8"))
+        dictionary = AmiDictionary.create_from_xml_object(xml_tree, ignorecase=ignorecase)
+        dictionary.xml_content = xml_tree
+        dictionary.file = xml_file
+        dictionary.root = xml_tree.getroot()
+
         return dictionary
 
     @classmethod
     def create_dictionary_from_words(cls, terms, title=None, desc=None, wikilangs=None, wikidata=False, outdir=None):
         """use raw list of words and optionally lookup each. choosing WD page and using languages
         :param terms: list of words/phrases
-        :param title: for dictionary oject and file
+        :param title: for dictionary object and file
         :param desc: description of dictionary
         :param wikilangs: languages to use in Wikidata default EN)
         :param wikidata: if true lookup wikidata
@@ -325,7 +496,7 @@ class AmiDictionary:
         for term in terms:
             term = term.strip()
             if term:
-                dup_entry = self.get_entry(term)
+                dup_entry = self.get_lxml_entry(term)
                 if dup_entry is None:
                     self._add_entry(term)
                 elif duplicates == "error":
@@ -421,15 +592,6 @@ class AmiDictionary:
         :return: new Dictionary"""
         return AmiDictionary.create_from_xml_file(file) if file is not None else None
 
-    @classmethod
-    def read_dictionary_from_xml_file(cls, file, ignorecase=True):
-        xml_tree = ET.parse(str(file), parser=ET.XMLParser(encoding="utf-8"))
-        dictionary = cls.create_from_xml_object(xml_tree, ignorecase=ignorecase)
-        dictionary.xml_content = xml_tree
-        dictionary.file = file
-        dictionary.root = xml_tree.getroot()
-        return dictionary
-
     #    class AmiDictionary:
 
     @classmethod
@@ -448,9 +610,7 @@ class AmiDictionary:
         dictionary.title = dictionary.root.xpath("@title")
         dictionary.ignorecase = ignorecase
         dictionary.entries = list(dictionary.root.findall(ENTRY))
-        print(f"dictionary.entries {len(dictionary.entries)}")
         dictionary.create_entry_by_term()
-        print(f"entry_by_term {len(dictionary.entry_by_term)}")
         dictionary.term_set = set()
         return dictionary
 
@@ -522,7 +682,7 @@ class AmiDictionary:
         return term.lower() if term is not None and self.ignorecase else term
 
     def get_xml_and_image_url(self, term):
-        entry = self.get_entry(term)
+        entry = self.get_lxml_entry(term)
         entry_xml = ET.tostring(entry)
         image_url = entry.find(".//" + IMAGE)
         return entry_xml, image_url.text if image_url is not None else None
@@ -558,7 +718,7 @@ class AmiDictionary:
 
     #    class AmiDictionary:
 
-    def get_entry(self, termx, ignorecase=True):
+    def get_lxml_entry(self, termx, ignorecase=True):
         """get entry if term attribute == term (case may matter
         :param termx: term to search for
         :param ignorecase: if true searches for any case variant
@@ -574,6 +734,7 @@ class AmiDictionary:
         entry = self.entry_by_term[lcase] if lcase in self.entry_by_term else None
         # if case-sensitive check whether term was different case
         if entry is not None:
+            assert type(entry) is _Element, f"found {type(entry)}"
             entry_term = entry.attrib[TERM]
             # if the term is case sensitive, compare them
             if not ignorecase and entry_term != termx:
@@ -587,17 +748,29 @@ class AmiDictionary:
         :param term: to search for
         :return: entry wrapped in AmiEntry
         """
-        entry = self.get_entry(term)
+        entry = self.get_lxml_entry(term)
         return None if entry is None else AmiEntry.create_from_element(entry)
 
     def get_entry_count(self):
         assert self.entry_by_term is not None
         return len(self.entry_by_term)
 
-    def get_entries(self):
+    def get_lxml_entries(self):
+        """
+        get lxml_entries from dictionary
+        :return: list of lxml_entry's
+        """
         assert self.entry_by_term
         return list(self.entry_by_term.values())
 
+    @classmethod
+    def get_term(cls, lxml_entry):
+        return lxml_entry.get(TERM)
+
+    @classmethod
+    def get_wikidata_id(cls, lxml_entry):
+        return lxml_entry.get(WIKIDATA_ID)
+
     def get_ami_entries(self):
         """creates entriws from elements in self.entry_by_term
         TODO maybe index the AmiEntry's instead
@@ -623,26 +796,32 @@ class AmiDictionary:
 
     def delete_entry_by_term(self, term):
         """removes an entry with given term"""
-        entry = self.get_entry(term)
+        entry = self.get_lxml_entry(term)
         if entry is not None:
             self.root.remove(entry)
             self.entry_by_term.pop(term)
 
     def create_and_add_entry_with_term(self, term, replace=True):
-        entry_new = AmiEntry.create_from_term(term)
-        entry_exist = self.get_entry(term)
+        """creates lxml_entry element from term
+        uses AmiEntry.create_lxml_entry_from_term(term)
+        :param term: term attribute
+        :param replace: if True, replace entry element
+        :return: new lxml_entry
+        """
+        lxml_entry_new = AmiEntry.create_lxml_entry_from_term(term)
+        lxml_entry_exist = self.get_lxml_entry(term)
 
-        if entry_exist is not None:
+        if lxml_entry_exist is not None:
             if replace:
-                entry_exist.addnext(entry_new)
-                entry_exist.getparent().remove(entry_exist)
+                lxml_entry_exist.addnext(lxml_entry_new)
+                lxml_entry_exist.getparent().remove(lxml_entry_exist)
             else:
                 raise AMIDictError("attempt to add duplicate entry with replace=False")
         else:
-            self.root.append(entry_new)
-        if entry_new is not None:
-            self.entry_by_term[term] = entry_new
-        return entry_new
+            self.root.append(lxml_entry_new)
+        if lxml_entry_new is not None:
+            self.entry_by_term[term] = lxml_entry_new
+        return lxml_entry_new
 
     def check_unique_wikidata_ids(self):
         # print("entries", len(self.entries))
@@ -747,13 +926,85 @@ class AmiDictionary:
 
         return wikidata_page
 
-    @classmethod
-    def get_term(cls, entry):
-        return entry.get(TERM)
+    def disambiguate_wikidata_by_desc(self, entry):
+        """
 
-    @classmethod
-    def get_wikidata_id(cls, entry):
-        return entry.get(WIKIDATA_ID)
+        """
+        wikidata_id = AmiDictionary.get_wikidata_id(entry)
+        if not AmiDictionary.is_valid_wikidata_id(wikidata_id):
+            term = AmiDictionary.get_term(entry)
+            self.lookup_and_add_wikidata_to_entry(entry, allowed_descriptions="")
+            wikidata_id = AmiDictionary.get_wikidata_id(entry)
+            if wikidata_id is None:
+                print(f"no wikidata entry for {term}")
+                entry.attrib[WIKIDATA_ID] = AmiDictionary.NOT_FOUND
+            else:
+                print(f"found {wikidata_id} for {term} desc = {entry.get('desc')}")
+
+    def get_disambiguated_raw_wikidata_ids(self):
+        """
+        returns disambiguated list of (term, idlist) tuples for ambiguous raw@wikidataIDs
+        iterates over all entry/raw@wikidataID attributes and disambiguates the IDs by matching
+        against the entry name
+        :return: list of (term, idlist) tuples if disambiguates
+
+        """
+        _entries = self.get_entries_with_raw_wikidata_ids()
+        ami_entries = AmiEntry.create_from_elements(_entries)
+        _term_id_list = []
+        for ami_entry in ami_entries:
+            wikidata_pages = ami_entry.get_wikidata_pages_from_raw_wikidata_ids_matching_wikidata_page_title()
+            ids = [page.get_id() for page in wikidata_pages]
+            if ids:
+                _term_id_list.append((ami_entry.get_term(), ids))
+        return _term_id_list
+
+    def lookup_missing_wikidata_ids(self, lookup_string=NAME, maxhits=99999):
+        """
+        finds entries with missing WikidataIDs and searches wikidata by name or term
+        creates WikidataLookup which holds hits_dict with details of possible match
+
+        e.g .
+        # {
+#     {'BECCS': {'Q146790': 'Aomori',
+#                'Q209727': 'palmitic acid',
+#                'Q455712': 'Domenico di Pace Beccafumi',
+#                'Q472237': 'Nikolaos Gyzis',
+#                'Q507854': 'Karlstadt am Main'},
+#      'CBEs': {'Q1391': 'Maryland',
+#               'Q227': 'Azerbaijan',
+#               'Q7024': 'Lugano',
+#               'Q8093': 'Nintendo',
+#               'Q884': 'South Korea'},
+#      'WMO': {'Property:P4136': 'WIGOS station ID',
+#              'Property:P5956': 'War Memorials Online ID',
+#              'Property:P9737': 'WMO code',
+#              'Q170424': 'World Meteorological Organization',
+#              'Q4468436': 'White Mountain Airport'}}
+#     }
+
+        :param lookup_string: either "name" (NAME) OR "term" (TERM); default NAME
+        :param maxhits: exit after maxhits entries
+        :return: new WikidataLookup with lookup.hits_dict
+        """
+        lxml_entries = self.get_lxml_entries_with_missing_wikidata_ids()
+        lookup = WikidataLookup()
+        for i, lxml_entry in enumerate(lxml_entries):
+            if i >= maxhits:
+                break
+            ami_entry = AmiEntry.create_from_element(lxml_entry)
+            # where only some of name/term are present
+            if lookup_string == NAME:
+                string = ami_entry.get_name()
+                if not string:
+                    string = ami_entry.get_term()
+            else:
+                string = ami_entry.get_term()
+                if not string:
+                    string = ami_entry.get_name()
+
+            lookup.get_possible_wikidata_hits(string)
+        return lookup
 
     def markup_html_from_dictionary(self, target_path, output_path, background_color):
         term_set = self.get_or_create_term_set()
@@ -783,7 +1034,7 @@ class AmiDictionary:
         match_counter = Counter()
         id_dict = dict()
         for a_elem in a_elems:
-            entry = self.get_entry(a_elem.text, ignorecase=True)  # lookup in dictionary
+            entry = self.get_lxml_entry(a_elem.text, ignorecase=True)  # lookup in dictionary
             if entry is None:
                 # print(f" BUG {text}") # this seemed to catch "Page n"
                 continue
@@ -880,7 +1131,7 @@ class AmiDictionary:
     def set_encoding(self, encoding=UTF_8):
         assert self.root is not None
         # self.root.docinfo.encoding = encoding
-        print(f"cannot yet set encoding")
+        # print(f"cannot yet set encoding")
 
     def set_version(self, version='0.0.1'):
         assert self.root is not None
@@ -894,16 +1145,16 @@ class AmiDictionary:
     def get_first_entry(self):
         """gets first entry mainly for testing
         """
-        if self.root is None or len(self.get_entries()) == 0:
+        if self.root is None or len(self.get_lxml_entries()) == 0:
             return None
-        return self.get_entries()[0]
+        return self.get_lxml_entries()[0]
 
     def get_first_ami_entry(self):
         """gets first entry mainly for testing
         """
-        if self.root is None or len(self.get_entries()) == 0:
+        if self.root is None or len(self.get_lxml_entries()) == 0:
             return None
-        return AmiEntry.create_from_element(self.get_entries()[0])
+        return AmiEntry.create_from_element(self.get_lxml_entries()[0])
 
     # TODO is this in the right place?
     @classmethod
@@ -942,7 +1193,7 @@ class AmiDictionary:
         """
         # print(f"looking for {term} in {len(self.get_entries())}")
         _entries = []
-        for entry in self.get_entries():
+        for entry in self.get_lxml_entries():
             _term = AmiEntry.get_attribute_value(entry, TERM)
             print(f"{type(entry)}  {_term}")
             if _term == term:
@@ -958,24 +1209,28 @@ class AmiDictionary:
         return list(self.entry_by_term.keys())
 
     @classmethod
-    def is_valid_version_string(cls, versionx):
+    def is_valid_version_string(cls, version):
         """tests validity of version string major.minor.patch
-
-        e.g. version = "1.2.3"
+        :param version: e.g. version = "1.2.3"
+        :raises AMIDictError: version is None (maybe missing attribute)
+        :raises AMIDictError: version does not have 3 parts")
+        :raises AMIDictError: version parts must be integers")
+        :returns: True (errors raises exceptions)
         """
-        if versionx is None:
-            raise AMIDictError(f"{cls} does not have version attribute ")
-        parts = versionx.split(".")
+        if version is None:
+            raise AMIDictError(f"{cls} version is None")
+        parts = version.split(".")
         if len(parts) != 3:
-            raise AMIDictError(f"{cls} version attribute {versionx} does not have 3 parts")
+            raise AMIDictError(f"{cls} version attribute {version} does not have 3 parts")
         try:
             for part in parts:
                 _ = int(part)
         except Exception:
-            raise AMIDictError(f"{cls} version attribute {versionx} parts must be integers")
+            raise AMIDictError(f"{cls} version attribute {version} parts must be integers")
         return True
 
     def check_validity(self):
+
         self.check_valid_attributes()
         self.check_valid_children()
 
@@ -996,6 +1251,32 @@ class AmiDictionary:
             else:
                 print(f"forbidden child of {self.root.tag}: {child.tag} ; allowed = {AmiDictionary.ALLOWED_CHILDREN}")
 
+    def get_lxml_entries_with_missing_wikidata_ids(self):
+        """
+        get all entries for which there is no wikidataId or its value is ""
+        :return: list of lxml_entries (may be empty)
+        """
+        lxml_entries = self.get_lxml_entries()
+        missing_wikidata_entries = []
+        for lxml_entry in lxml_entries:
+            wikidata_id = AmiEntry.create_from_element(lxml_entry).get_wikidata_id()
+            if not wikidata_id or wikidata_id.strip() == "":
+                missing_wikidata_entries.append(lxml_entry)
+        return missing_wikidata_entries
+
+    def get_entries_with_raw_wikidata_ids(self):
+        """
+        get all entries for which there is a raw@wikidataID child
+        :return: list of entries (may be empty)
+        """
+        entries = self.get_lxml_entries()
+        missing_wikidata_entries = []
+        for entry in entries:
+            wikidata_id = AmiEntry.create_from_element(entry).get_wikidata_id()
+            if not wikidata_id or wikidata_id.strip() == "":
+                missing_wikidata_entries.append(entry)
+        return missing_wikidata_entries
+
 
 class AmiSynonym:
     TAG = "synonym"
