@@ -1,12 +1,16 @@
 import json
 import logging
 import os
+from enum import Enum
 
 from lxml import etree as ET
 from lxml import etree, html
 import lxml.etree
 
 # local
+from py4ami.util import Util
+
+# from py4ami.ami_dict import REGEX_BLACKLIST
 
 logging.debug("loading wikimedia.py")
 
@@ -43,6 +47,10 @@ ID_NAME = "id_name"
 SPQ_NAME = "sparql_name"
 DICT_NAME = "dict_name"
 
+REGEX_BLACKLIST = [".*((scientific|journal)\\s+)?article.*",
+                   ".*((scientific|academic)\\s+)?journal.*",
+                   ".*(film|song|album)"]
+
 
 # this should go in config files
 class WikidataPredicate:
@@ -50,7 +58,7 @@ class WikidataPredicate:
         "P31": {
             "Q11173": "chemical compound",
             "weight": 0.9
-            },
+        },
 
         "P117": {"object": "chemical structure",
                  "weight": 0.9
@@ -61,12 +69,10 @@ class WikidataPredicate:
         "P223": {"object": "canonical SMILES",
                  "weight": 0.9
                  },
-        "P223": {"object": "canonical SMILES",
-                 "weight": 0.9
-                 },
         # "IDENTIFIER": {"InChI" : }
     }
 
+
 # TODO add docstrings and check return values
 class WikidataLookup:
 
@@ -75,6 +81,7 @@ class WikidataLookup:
         self.wikidata_dict = None
         self.root = None
         self.exact_lookup = exact_lookup
+        self.hits_dict = dict()
 
     def lookup_wikidata(self, term):
         """
@@ -83,40 +90,41 @@ class WikidataLookup:
         NOTE requires Internet
 
         :param term: word or phrase to lookup
-        :return: triple (e.g. qitem[0], qitem[1]["desc"], wikidata_hits)
+        :return: triple (e.g. hit0_id, hit0_description, wikidata_hits)
         """
 
         from urllib.parse import quote
 
+        if not term:
+            logging.warning("null term")
+            return None, None, None
         self.term = term
+        MAX_ENTRIES = 5
         url = WIKIDATA_QUERY_URI + quote(term.encode('utf8'))
-        print(f"url {url}")
+        # print(f"url {url}")
         self.root = ParserWrapper.parse_utf8_html_to_root(url)
         body = self.root.find(BODY)
         ul = body.find(".//ul[@class='" + MW_SEARCH_RESULTS + "']")
-        qitem = None  # to avoid UnboundLocalError
+        hit0 = None  # to avoid UnboundLocalError
         if ul is not None:
             self.wikidata_dict = self.create_dict_for_all_possible_wd_matches(ul)
             if len(self.wikidata_dict) == 0:
                 assert False, f"no wikidata hits for {term}"
             sort_orders = sorted(self.wikidata_dict.items(), key=lambda item: int(item[1][STATEMENTS]), reverse=True)
             if sort_orders:
-                wikidata_hits = [s[0] for s in sort_orders[:5]]
-                #            pprint.pprint(sort_orders[0:3])
+                wikidata_hits = [s[0] for s in sort_orders[:MAX_ENTRIES]]
                 #  take the first
-                qitem = sort_orders[0]
+                hit0 = sort_orders[0]
             else:
                 print(f"no wikidata hits for {term}")
 
                 # TODO fix non-tuples
-        if qitem is None:
+        if hit0 is None:
             return None, None, None
         else:
-            return qitem[0], qitem[1]["desc"], wikidata_hits
-
-    # def lookup_qitem(self, qitem):
-    #     root = ParserWrapper.parse_utf8_html_to_root(WIKIDATA_SITE + qitem)
-    #     return root
+            hit0_id = hit0[0]
+            hit0_description = hit0[1]["desc"]
+            return hit0_id, hit0_description, wikidata_hits
 
     def lookup_items(self, terms):
         """looks up a series of terms and returns a tuple of list(qitem), list(desc)
@@ -158,6 +166,26 @@ class WikidataLookup:
         sub_dict[STATEMENTS] = \
             li.find("./div[@class='" + MW_SEARCH_RESULT_DATA + "']").text.split(",")[0].split(" statement")[0]
 
+    def get_possible_wikidata_hits(self, name):
+        entry_hits = self.lookup_wikidata(name)
+        print(f"------{name}-------")
+        if not entry_hits[0]:
+            #                print(f" no hit for {name}")
+            pass
+        else:
+            hits = dict()
+            for qid in entry_hits[2]:
+                wpage = WikidataPage(qid)
+                description = wpage.get_description()
+                regex = Util.matches_regex_list(description, REGEX_BLACKLIST)
+                if regex:
+                    logging.debug(f"{regex} // {description}")
+                else:
+                    logging.debug(f"\n{wpage.get_title()}\n{description}")
+                    hits[qid] = wpage.get_title()
+            if hits:
+                self.hits_dict[name] = hits
+
 
 class WikidataBrowser:
     """ """
@@ -209,6 +237,7 @@ class WikidataBrowser:
         for subelem in element.xpath(xpath):
             subelem.getparent().remove(subelem)
 
+
 class WikidataFilter:
 
     @classmethod
@@ -227,7 +256,6 @@ class WikidataFilter:
         return filter
 
 
-
 class WikidataProperty:
 
     def __init__(self):
@@ -243,11 +271,11 @@ class WikidataProperty:
     @classmethod
     def create_property_from_element(cls, html_element):
         """Create from HTML element from wikidata.org page"""
-        property = None
+        propertyx = None
         if html_element is not None:
-            property = WikidataProperty()
-            property.element = html_element
-        return property
+            propertyx = WikidataProperty()
+            propertyx.element = html_element
+        return propertyx
 
     @property
     def id(self):
@@ -297,14 +325,19 @@ class WikidataProperty:
         if not property_list:
             return properties_dict
 
-        for property in property_list:
-            property_dict = property.create_property_dict()
-            properties_dict[property.id] = property_dict
+        for propertyx in property_list:
+            property_dict = propertyx.create_property_dict()
+            properties_dict[propertyx.id] = property_dict
         return properties_dict
 
+
 # class WikidataProperty:
 
 
+def label_match(label, wikidata_label, method, ignorecase):
+    raise NotImplemented("label match")
+
+
 class WikidataPage:
     PROPERTY_ID = "id"
 
@@ -332,19 +365,23 @@ class WikidataPage:
         wikidata_page.json = response.json()
         return wikidata_page
 
-    def get_root_for_item(self, qitem):
+    def get_root_for_item(self, pqitem):
         """search wikidata site for QItem
-        :param qitem: Qitem (or P item to search
+        :param pqitem: Qitem (or P item to search
         :return: parsed lxml root"""
         if self.root is None:
-            self.root = ParserWrapper.parse_utf8_html_to_root(self.get_wikidata_site() + qitem)
+            url_for_pqitem = self.get_url_for_pqitem(pqitem)
+            self.root = ParserWrapper.parse_utf8_html_to_root(url_for_pqitem)
         return self.root
 
+    def get_url_for_pqitem(self, qitem):
+        return self.get_wikidata_site() + qitem
+
     @classmethod
     def get_wikidata_site(cls):
         return WIKIDATA_SITE
 
-# WikidataPage
+    # WikidataPage
 
     def get_wikipedia_page_links(self, lang_list):
         """
@@ -433,7 +470,8 @@ class WikidataPage:
         """
         selector = WikidataPage.get_data_property_xpath()
         property_element_list = self.root.xpath(selector)
-        property_list = [WikidataProperty.create_property_from_element(p_element) for p_element in property_element_list]
+        property_list = [WikidataProperty.create_property_from_element(p_element) for p_element in
+                         property_element_list]
         return property_list
 
     @classmethod
@@ -445,7 +483,7 @@ class WikidataPage:
         """get list of ids presenting properties
         These will be in the left-hand column of the page
         :return: ids , example ['P31', 'P279', 'P361', 'P117']"""
-        property_list = self.get_data_property_list() # WikidataProperty
+        property_list = self.get_data_property_list()  # WikidataProperty
         property_id_list = [property.id for property in property_list]
         return property_id_list
 
@@ -470,8 +508,6 @@ class WikidataPage:
             qvals = hdiv_p[0].xpath(".//div[@class='wikibase-snakview-body']//a[starts-with(@title,'Q')]")
         return qvals
 
-
-
     @classmethod
     def get_predicate_object_from_file(cls, wikidata_file, pred, obj):
         """finds wikidata predicate+object
@@ -532,6 +568,20 @@ class WikidataPage:
         desc = "" if not desc_list else desc_list[0].text
         return desc
 
+    def get_id(self):
+        """
+        get id from <span class="wikibase-title-id">(Q42)</span>
+        remove brackets
+        :return: P/Q id or None
+        """
+        id_list = self.get_elements_for_normalized_attrib_val("class", "wikibase-title-id")
+        id = None if not id_list else id_list[0].text
+        if id:
+            id = id.strip("(").strip(")")
+        else:
+            id = None
+        return id
+
     def get_elements_for_normalized_attrib_val(self, attname, attval, lead="//*", trail=""):
         """some attvals contain leading/trailing space
         searches for <lead>/*[@foo=bar] where bar might be whitespaced
@@ -587,15 +637,57 @@ class WikidataPage:
         xpath = f"{lead}[contains(concat(' ',normalize-space(@{attname}),' '),concat(' ',normalize-space('{attval}'),' '))]{trail}"
         return xpath
 
-    # def _get_elements_for_normalized_attrib_val(self, attname, attval):
-    #     """some attvals contain trailing space"""
-    #     xpath = f"/html//*[contains(concat(' ',@{attname},' '),concat(' ','{attval}',' '))]"
-    #     print(f"xpath |{xpath}|")
-    #     xpath = f"/html//*[normalize-space(@{attname})='{attval}']"
-    #     elems = self.root.xpath(xpath)
-    #     return elems
+    # WikidataPage
+
+    def title_matches(self, wikidata_title, ignorecase=True):
+        """
+        does the wikidata_title match the title of the psge
+        """
+        if not wikidata_title:
+            return False
+        title = self.get_title()
+        if not title:
+            return False
+        if ignorecase:
+            title = title.lower()
+            wikidats_title = wikidata_title.lower()
+        return title == wikidata_title
+
+    class WikidataLabelMatch(Enum):
+        EXACT = 1,
+        SUBSTRING = 2,
+
+    def find_name_to_wikidata_match(labels=None,
+                                    ids=None,
+                                    wikidata_label=None,
+                                    method=WikidataLabelMatch.EXACT,
+                                    ignorecase=True,
+                                    ):
+        """iterate over list of wikidata titles/labels to find closest/exact match for title
+        :param titles: list of previously extracted wikidata labels
+        NOT YET USED
+        """
+        if labels == None:
+            logging.info(f"no labels given")
+            return None
+        if ids == None:
+            logging.error(f"must give list of ids")
+            return None
+        if len(labels) != len(ids):
+            logging.error(f"labels {labels} and ids {ids} are different lengths")
+            return None
+        if wikidata_label == None:
+            logging.error(f"must give required wikidata label")
+            return None
+        idlist = []
+        for label, id in zip(labels, ids):
+            if label_match(label, wikidata_label, method, ignorecase):  # METHOD NOT WRITTEN
+                idlist.append(id)
+
 
 """<div class="wikibase-entitytermsview-heading-description">chemical compound</div>"""
+
+
 class WikidataSparql:
 
     def __init__(self, dictionary):
@@ -702,9 +794,11 @@ class ParserWrapper:
         root = tree.getroot()
         return root
 
+
 import pprint
 import requests
 
+
 class WikidataExtractor:
     """Thanks to Awena for showing the approach"""
     VERSION = "0.1"
@@ -829,4 +923,3 @@ class WikidataExtractor:
         # 			if key=="P31":		# instance of
         # 				result["instance"]			= data["claims"][key][0]["mainsnak"]["datavalue"]["value"]["id"]
         return result
-
