@@ -1,4 +1,6 @@
 # Tests wikipedia and wikidata methods under pytest
+import ast
+import lxml
 import os
 import pprint
 import unittest
@@ -8,7 +10,8 @@ from lxml import etree, html
 import requests
 # local
 from py4ami.wikimedia import WikidataPage, ParserWrapper, WikidataExtractor, WikidataProperty, WikidataFilter
-from py4ami.ami_dict import WIKIDATA_ID
+from py4ami.ami_dict import WIKIDATA_ID, AmiEntry
+from test.resources import Resources
 
 """This runs under Pycharm and also
 cd pyami # toplevel checkout
@@ -91,9 +94,9 @@ class TestWikidataLookup(unittest.TestCase):
         wikidata_lookup = WikidataLookup()
         # qitems, descs = wikidata_lookup.lookup_items(terms)
         temp_dir = Path(Path(__file__).parent.parent, "temp")
-        dictfile, amidict = AMIDict.create_from_list_of_strings_and_write_to_file(terms, title="parkinsons",
-                                                                                  wikidata=True, directory=temp_dir)
-        assert os.path.exists(dictfile)
+        # dictfile, amidict = AMIDict.create_from_list_of_strings_and_write_to_file(terms, title="parkinsons",
+        #                                                                           wikidata=True, directory=temp_dir)
+        # assert os.path.exists(dictfile)
 
     def test_parse_wikidata_html(self):
         """find Wikidata items with given property
@@ -380,10 +383,10 @@ class TestWikidataLookup(unittest.TestCase):
         path = Path(EO_COMPOUND_DIR, "dict_5.xml")
         dictionary = AmiDictionary.create_from_xml_file(str(path))
         assert len(dictionary.entries) == 5
-        entry = dictionary.get_entry("allylhexanoate")
+        entry = dictionary.get_lxml_entry("allylhexanoate")
         assert entry.get(WIKIDATA_ID) == "Q3270746"
 
-        entry = dictionary.get_entry("allyl isovalerate")
+        entry = dictionary.get_lxml_entry("allyl isovalerate")
         assert entry.get(WIKIDATA_ID) is None
         dictionary.lookup_and_add_wikidata_to_entry(entry)
         assert entry.get(WIKIDATA_ID) == "Q27155908"
@@ -419,7 +422,8 @@ class TestWikidataLookup(unittest.TestCase):
         dictionary.write_to_file(Path(output_dir, "eoCompound", "eoCompound1.xml"))
 
     def test_disambiguation(self):
-        """attempts to disambiguate the result of PMR-wikidata lookup"""
+        """attempts to disambiguate the result of PMR-wikidata lookup
+        """
         input_dir = EO_COMPOUND_DIR
         output_dir = TEMP_DIR
         input_path = Path(input_dir, "disambig.xml")
@@ -432,19 +436,61 @@ class TestWikidataLookup(unittest.TestCase):
         }
         dictionary = AmiDictionary.create_from_xml_file(str(input_path))
         for entry in dictionary.entries:
-            wikidata_id = AmiDictionary.get_wikidata_id(entry)
-            if not AmiDictionary.is_valid_wikidata_id(wikidata_id):
-                term = AmiDictionary.get_term(entry)
-                # print(f"no wikidataID in entry: {term}")
-                dictionary.lookup_and_add_wikidata_to_entry(entry, allowed_descriptions="")
-                wikidata_id = AmiDictionary.get_wikidata_id(entry)
-                if wikidata_id is None:
-                    print(f"no wikidata entry for {term}")
-                    entry.attrib[WIKIDATA_ID] = AmiDictionary.NOT_FOUND
-                else:
-                    print(f"found {wikidata_id} for {term} desc = {entry.get('desc')}")
+            dictionary.disambiguate_wikidata_by_desc(entry)
         dictionary.write_to_file(Path(output_dir, "eoCompound", "disambig.xml"))
 
+    def test_extract_multiple_wikidata_hits_ghg(self):
+        """
+        test multiple hits for 'GHG' and use heuristics to find the most likely
+        uses dictionary created from abbreviations via docanalysis
+        requires WIKIDATA LOOKUP on Internet
+
+        ghg entry is:
+  <entry term="GHG" name="Greenhouse Gas" >
+    <raw wikidataID="['Q167336', 'Q3588927', 'Q925312', 'Q57584895', 'Q110612403', 'Q112192791', 'Q140182']"/>
+  </entry>
+
+        """
+
+        ami_dict = AmiDictionary.create_from_xml_file(Resources.IPCC_CHAP02_ABB_DICT)
+        assert ami_dict is not None
+        ghg = "GHG"
+        ghg_element = ami_dict.get_lxml_entry(ghg)
+        assert ghg_element, f"entry for {ghg} is None, probably not found"
+        assert type(ghg_element) is lxml.etree._Element, f"entry has type {type(ghg_element)}"
+        ghg_entry = AmiEntry.create_from_element(ghg_element)
+        assert type(ghg_entry) is AmiEntry, f"ami_entry is type {type(ghg_entry)}"
+        assert ghg_entry is not None
+        wikidata_id = ghg_entry.wikidata_id
+        assert not wikidata_id
+        raw_wikidata_ids = ghg_entry.get_raw_child_wikidata_ids()
+        assert len(raw_wikidata_ids) == 7
+        assert raw_wikidata_ids == [
+            'Q167336',
+            'Q3588927',
+            'Q925312',
+            'Q57584895',
+            'Q110612403',
+            'Q112192791',
+            'Q140182'
+        ]
+
+    def test_get_best_match_for_ghg(self):
+        """
+        gets best wikidata match for term
+        WORKING
+entry like:
+  <entry term="GHG" name="Greenhouse Gas" >
+    <raw wikidataID="['Q167336', 'Q3588927', 'Q925312', 'Q57584895', 'Q110612403', 'Q112192791', 'Q140182']"/>
+  </entry>
+
+        """
+        ami_dict = AmiDictionary.create_from_xml_file(Resources.IPCC_CHAP02_ABB_DICT)
+        term = "GHG"
+        ami_entry = ami_dict.get_ami_entry(term)
+        matched_pages = ami_entry.get_wikidata_pages_from_raw_wikidata_ids_matching_wikidata_page_title()
+        assert len(matched_pages) == 1 and matched_pages[0].get_id() == "Q167336"
+
     def test_get_instances(self):
         """<div class="wikibase-statementview-mainsnak-container">
 <div class="wikibase-statementview-mainsnak" dir="auto"><div class="wikibase-snakview wikibase-snakview-e823b98d1498aa78e139709b1b02f5decd75c887">
